@@ -22,7 +22,8 @@
 
 #include "gdkclipboard.h"
 #include "gdkcontentformats.h"
-#include "gdkintl.h"
+#include <glib/gi18n-lib.h>
+#include "gdkprivate.h"
 
 /**
  * GdkContentProvider:
@@ -197,7 +198,7 @@ gdk_content_provider_class_init (GdkContentProviderClass *class)
    * Emitted whenever the content provided by this provider has changed.
    */
   signals[CONTENT_CHANGED] =
-    g_signal_new ("content-changed",
+    g_signal_new (I_("content-changed"),
                   G_TYPE_FROM_CLASS (class),
                   G_SIGNAL_RUN_LAST,
                   G_STRUCT_OFFSET (GdkContentProviderClass, content_changed),
@@ -278,10 +279,6 @@ gdk_content_provider_content_changed (GdkContentProvider *provider)
  * Asynchronously writes the contents of @provider to @stream in the given
  * @mime_type.
  *
- * When the operation is finished @callback will be called. You must then call
- * [method@Gdk.ContentProvider.write_mime_type_finish] to get the result
- * of the operation.
- *
  * The given mime type does not need to be listed in the formats returned by
  * [method@Gdk.ContentProvider.ref_formats]. However, if the given `GType` is
  * not supported, `G_IO_ERROR_NOT_SUPPORTED` will be reported.
