@@ -25,6 +25,7 @@
 #include "gdkdisplay.h"
 #include "gdkenumtypes.h"
 #include "gdkdmabuftextureprivate.h"
+#include "gdkdmabuftexturebuilderprivate.h"
 
 #include <cairo-gobject.h>
 
@@ -57,7 +58,7 @@ struct _GdkDmabufTextureBuilderClass
  *
  * DMA buffers are commonly called **_dma-bufs_**.
  *
- * DMA buffers a feature of the Linux kernel to enable efficient buffer and
+ * DMA buffers are a feature of the Linux kernel to enable efficient buffer and
  * memory sharing between hardware such as codecs, GPUs, displays, cameras and the
  * kernel drivers controlling them. For example, a decoder may want its output to
  * be directly shared with the display server for rendering without a copy.
@@ -67,7 +68,7 @@ struct _GdkDmabufTextureBuilderClass
  *
  * The memory that is shared via DMA buffers is usually stored in non-system memory
  * (maybe in device's local memory or something else not directly accessible by the
- * CPU), and accessing this memory from the CPU may have higher than usual overhead.
+ * CPU), and accessing this memory from the CPU may have higher-than-usual overhead.
  *
  * In particular for graphics data, it is not uncommon that data consists of multiple
  * separate blocks of memory, for example one block for each of the red, green and
@@ -87,7 +88,7 @@ struct _GdkDmabufTextureBuilderClass
  *
  * For historical reasons, some producers of dma-bufs don't provide an explicit modifier, but
  * instead return `DMA_FORMAT_MOD_INVALID` to indicate that their modifier is **_implicit_**.
- * GTK tries to accomodate this situation by accepting `DMA_FORMAT_MOD_INVALID` as modifier.
+ * GTK tries to accommodate this situation by accepting `DMA_FORMAT_MOD_INVALID` as modifier.
  *
  * The operation of `GdkDmabufTextureBuilder` is quite simple: Create a texture builder,
  * set all the necessary properties, and then call [method@Gdk.DmabufTextureBuilder.build]
@@ -948,22 +949,20 @@ gdk_dmabuf_texture_builder_set_update_region (GdkDmabufTextureBuilder *self,
  *
  * Builds a new `GdkTexture` with the values set up in the builder.
  *
- * It is a programming error to call this function if any mandatory
- * property has not been set.
+ * It is a programming error to call this function if any mandatory property has not been set.
  *
- * If the dmabuf is not supported by GTK, %NULL will be returned and @error will be set.
+ * Not all formats defined in the `drm_fourcc.h` header are supported. You can use
+ * [method@Gdk.Display.get_dmabuf_formats] to get a list of supported formats. If the
+ * format is not supported by GTK, %NULL will be returned and @error will be set.
  *
  * The `destroy` function gets called when the returned texture gets released.
  *
- * It is possible to call this function multiple times to create multiple textures,
- * possibly with changing properties in between.
- *
  * It is the responsibility of the caller to keep the file descriptors for the planes
  * open until the created texture is no longer used, and close them afterwards (possibly
  * using the @destroy notify).
  *
- * Not all formats defined in the `drm_fourcc.h` header are supported. You can use
- * [method@Gdk.Display.get_dmabuf_formats] to get a list of supported formats.
+ * It is possible to call this function multiple times to create multiple textures,
+ * possibly with changing properties in between.
  *
  * Returns: (transfer full) (nullable): a newly built `GdkTexture` or `NULL`
  *   if the format is not supported
@@ -1004,3 +1003,19 @@ gdk_dmabuf_texture_builder_get_dmabuf (GdkDmabufTextureBuilder *self)
 {
   return &self->dmabuf;
 }
+
+void
+gdk_dmabuf_texture_builder_set_dmabuf (GdkDmabufTextureBuilder *self,
+                                       const GdkDmabuf         *dmabuf)
+{
+  gdk_dmabuf_texture_builder_set_fourcc (self, dmabuf->fourcc);
+  gdk_dmabuf_texture_builder_set_modifier (self, dmabuf->modifier);
+  gdk_dmabuf_texture_builder_set_n_planes (self, dmabuf->n_planes);
+
+  for (unsigned int i = 0; i < dmabuf->n_planes; i++)
+    {
+      gdk_dmabuf_texture_builder_set_fd (self, i, dmabuf->planes[i].fd);
+      gdk_dmabuf_texture_builder_set_stride (self, i, dmabuf->planes[i].stride);
+      gdk_dmabuf_texture_builder_set_offset (self, i, dmabuf->planes[i].offset);
+    }
+}
