@@ -103,6 +103,15 @@
 
 #define DEFAULT_ALLOWED_APIS GDK_GL_API_GL | GDK_GL_API_GLES
 
+static const GdkDebugKey gdk_gl_feature_keys[] = {
+  { "debug", GDK_GL_FEATURE_DEBUG, "GL_KHR_debug" },
+  { "unpack-subimage", GDK_GL_FEATURE_UNPACK_SUBIMAGE, "GL_EXT_unpack_subimage" },
+  { "half-float", GDK_GL_FEATURE_VERTEX_HALF_FLOAT, "GL_OES_vertex_half_float" },
+  { "sync", GDK_GL_FEATURE_SYNC, "GL_ARB_sync" },
+  { "base-instance", GDK_GL_FEATURE_BASE_INSTANCE, "GL_ARB_base_instance" },
+  { "buffer-storage", GDK_GL_FEATURE_BUFFER_STORAGE, "GL_EXT_buffer_storage" },
+};
+
 typedef struct _GdkGLContextPrivate GdkGLContextPrivate;
 
 struct _GdkGLContextPrivate
@@ -112,13 +121,9 @@ struct _GdkGLContextPrivate
 
   GdkGLMemoryFlags memory_flags[GDK_MEMORY_N_FORMATS];
 
-  guint has_khr_debug : 1;
+  GdkGLFeatures features;
   guint use_khr_debug : 1;
-  guint has_half_float : 1;
-  guint has_sync : 1;
-  guint has_unpack_subimage : 1;
   guint has_debug_output : 1;
-  guint has_bgra : 1;
   guint extensions_checked : 1;
   guint debug_enabled : 1;
   guint forward_compatible : 1;
@@ -147,6 +152,13 @@ enum {
 
 static GParamSpec *properties[LAST_PROP] = { NULL, };
 
+/**
+ * gdk_gl_error_quark:
+ *
+ * Registers an error quark for [class@Gdk.GLContext] errors.
+ *
+ * Returns: the error quark
+ **/
 G_DEFINE_QUARK (gdk-gl-error-quark, gdk_gl_error)
 
 G_DEFINE_ABSTRACT_TYPE_WITH_PRIVATE (GdkGLContext, gdk_gl_context, GDK_TYPE_DRAW_CONTEXT)
@@ -290,7 +302,7 @@ gdk_gl_context_create_egl_context (GdkGLContext *context,
   GdkDisplay *display = gdk_gl_context_get_display (context);
   EGLDisplay egl_display = gdk_display_get_egl_display (display);
   GdkGLContext *share = gdk_display_get_gl_context (display);
-  GdkGLContextPrivate *share_priv = gdk_gl_context_get_instance_private (share);
+  GdkGLContextPrivate *share_priv = share ? gdk_gl_context_get_instance_private (share) : NULL;
   EGLConfig egl_config;
   EGLContext ctx;
   EGLint context_attribs[N_EGL_ATTRS], i = 0, flags = 0;
@@ -898,11 +910,12 @@ gdk_gl_context_label_object_printf  (GdkGLContext *context,
 
 
 gboolean
-gdk_gl_context_has_unpack_subimage (GdkGLContext *context)
+gdk_gl_context_has_feature (GdkGLContext  *self,
+                            GdkGLFeatures  feature)
 {
-  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (self);
 
-  return priv->has_unpack_subimage;
+  return (priv->features & feature) == feature;
 }
 
 static gboolean
@@ -1664,10 +1677,53 @@ gdk_gl_version_init_epoxy (GdkGLVersion *version)
   *version = GDK_GL_VERSION_INIT (epoxy_version / 10, epoxy_version % 10);
 }
 
+static GdkGLFeatures
+gdk_gl_context_check_features (GdkGLContext *context)
+{
+  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+  GdkGLFeatures features = 0;
+
+  if (gdk_gl_context_get_use_es (context))
+    {
+      if (gdk_gl_version_greater_equal (&priv->gl_version, &GDK_GL_VERSION_INIT (3, 0)) ||
+          epoxy_has_gl_extension ("GL_EXT_unpack_subimage"))
+        features |= GDK_GL_FEATURE_UNPACK_SUBIMAGE;
+    }
+  else
+    {
+      features |= GDK_GL_FEATURE_UNPACK_SUBIMAGE;
+    }
+
+  if (epoxy_has_gl_extension ("GL_KHR_debug"))
+    features |= GDK_GL_FEATURE_DEBUG;
+
+  if (gdk_gl_context_check_version (context, "3.0", "3.0") ||
+      epoxy_has_gl_extension ("GL_OES_vertex_half_float"))
+    features |= GDK_GL_FEATURE_VERTEX_HALF_FLOAT;
+
+  if (gdk_gl_context_check_version (context, "3.2", "3.0") ||
+      epoxy_has_gl_extension ("GL_ARB_sync") ||
+      epoxy_has_gl_extension ("GL_APPLE_sync"))
+    features |= GDK_GL_FEATURE_SYNC;
+
+  if (gdk_gl_context_check_version (context, "4.2", "9.9") ||
+      epoxy_has_gl_extension ("GL_EXT_base_instance") ||
+      epoxy_has_gl_extension ("GL_ARB_base_instance"))
+    features |= GDK_GL_FEATURE_BASE_INSTANCE;
+
+  if (gdk_gl_context_check_version (context, "4.4", "9.9") ||
+      epoxy_has_gl_extension ("GL_EXT_buffer_storage") ||
+      epoxy_has_gl_extension ("GL_ARB_buffer_storage"))
+    features |= GDK_GL_FEATURE_BUFFER_STORAGE;
+
+  return features;
+}
+
 static void
 gdk_gl_context_check_extensions (GdkGLContext *context)
 {
   GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (context);
+  GdkGLFeatures supported_features, disabled_features;
   gboolean gl_debug = FALSE;
   GdkDisplay *display;
 
@@ -1685,69 +1741,54 @@ gdk_gl_context_check_extensions (GdkGLContext *context)
 
   if (priv->has_debug_output && gl_debug)
     {
-      gdk_gl_context_make_current (context);
       glEnable (GL_DEBUG_OUTPUT);
       glEnable (GL_DEBUG_OUTPUT_SYNCHRONOUS);
       glDebugMessageCallback (gl_debug_message_callback, NULL);
     }
 
-  if (gdk_gl_context_get_use_es (context))
-    {
-      priv->has_unpack_subimage = gdk_gl_version_greater_equal (&priv->gl_version, &GDK_GL_VERSION_INIT (3, 0)) ||
-                                  epoxy_has_gl_extension ("GL_EXT_unpack_subimage");
-      priv->has_khr_debug = epoxy_has_gl_extension ("GL_KHR_debug");
-      priv->has_bgra = epoxy_has_gl_extension ("GL_EXT_texture_format_BGRA8888");
-    }
-  else
-    {
-      priv->has_unpack_subimage = TRUE;
-      priv->has_khr_debug = epoxy_has_gl_extension ("GL_KHR_debug");
-      priv->has_bgra = TRUE;
+  /* If we asked for a core profile, but didn't get one, we're in legacy mode */
+  if (!gdk_gl_context_get_use_es (context) &&
+      !gdk_gl_version_greater_equal (&priv->gl_version, &GDK_GL_VERSION_INIT (3, 2)))
+    priv->is_legacy = TRUE;
 
-      /* We asked for a core profile, but we didn't get one, so we're in legacy mode */
-      if (!gdk_gl_version_greater_equal (&priv->gl_version, &GDK_GL_VERSION_INIT (3, 2)))
-        priv->is_legacy = TRUE;
-    }
+  supported_features = gdk_gl_context_check_features (context);
+  disabled_features = gdk_parse_debug_var ("GDK_GL_DISABLE",
+                                           gdk_gl_feature_keys,
+                                           G_N_ELEMENTS (gdk_gl_feature_keys));
 
-  if (priv->has_khr_debug && gl_debug)
+  priv->features = supported_features & ~disabled_features;
+
+  gdk_gl_context_init_memory_flags (context);
+
+  if ((priv->features & GDK_GL_FEATURE_DEBUG) && gl_debug)
     {
       priv->use_khr_debug = TRUE;
       glGetIntegerv (GL_MAX_LABEL_LENGTH, &priv->max_debug_label_length);
     }
 
-  priv->has_half_float = gdk_gl_context_check_version (context, "3.0", "3.0") ||
-                         epoxy_has_gl_extension ("GL_OES_vertex_half_float");
-
-  priv->has_sync = gdk_gl_context_check_version (context, "3.2", "3.0") ||
-                   epoxy_has_gl_extension ("GL_ARB_sync") ||
-                   epoxy_has_gl_extension ("GL_APPLE_sync");
-
-  gdk_gl_context_init_memory_flags (context);
-
-  {
-    int max_texture_size;
-    glGetIntegerv (GL_MAX_TEXTURE_SIZE, &max_texture_size);
-    GDK_DISPLAY_DEBUG (gdk_draw_context_get_display (GDK_DRAW_CONTEXT (context)), OPENGL,
-                       "%s version: %d.%d (%s)\n"
-                       "* GLSL version: %s\n"
-                       "* Max texture size: %d\n"
-                       "* Extensions checked:\n"
-                       " - GL_KHR_debug: %s\n"
-                       " - GL_EXT_unpack_subimage: %s\n"
-                       " - half float: %s\n"
-                       " - sync: %s\n"
-                       " - bgra: %s",
-                       gdk_gl_context_get_use_es (context) ? "OpenGL ES" : "OpenGL",
-                       gdk_gl_version_get_major (&priv->gl_version), gdk_gl_version_get_minor (&priv->gl_version),
-                       priv->is_legacy ? "legacy" : "core",
-                       glGetString (GL_SHADING_LANGUAGE_VERSION),
-                       max_texture_size,
-                       priv->has_khr_debug ? "yes" : "no",
-                       priv->has_unpack_subimage ? "yes" : "no",
-                       priv->has_half_float ? "yes" : "no",
-                       priv->has_sync ? "yes" : "no",
-                       priv->has_bgra ? "yes" : "no");
-  }
+  if (GDK_DISPLAY_DEBUG_CHECK (display, OPENGL))
+    {
+      int i, max_texture_size;
+      glGetIntegerv (GL_MAX_TEXTURE_SIZE, &max_texture_size);
+      gdk_debug_message ("%s version: %d.%d (%s)\n"
+                         "* GLSL version: %s\n"
+                         "* Max texture size: %d\n",
+                         gdk_gl_context_get_use_es (context) ? "OpenGL ES" : "OpenGL",
+                         gdk_gl_version_get_major (&priv->gl_version), gdk_gl_version_get_minor (&priv->gl_version),
+                         priv->is_legacy ? "legacy" : "core",
+                         glGetString (GL_SHADING_LANGUAGE_VERSION),
+                         max_texture_size);
+      gdk_debug_message ("Enabled features (use GDK_GL_DISABLE env var to disable):");
+      for (i = 0; i < G_N_ELEMENTS (gdk_gl_feature_keys); i++)
+        {
+          gdk_debug_message ("    %s: %s",
+                             gdk_gl_feature_keys[i].key,
+                             (priv->features & gdk_gl_feature_keys[i].value) ? "YES" :
+                             ((disabled_features & gdk_gl_feature_keys[i].value) ? "disabled via env var" :
+                             (((supported_features & gdk_gl_feature_keys[i].value) == 0) ? "not supported" :
+                             "Hum, what? This should not happen.")));
+        }
+    }
 
   priv->extensions_checked = TRUE;
 }
@@ -1922,8 +1963,6 @@ gdk_gl_context_get_glsl_version_string (GdkGLContext *self)
         return "#version 310 es";
       else if (gdk_gl_version_greater_equal (&priv->gl_version, &GDK_GL_VERSION_INIT (3, 0)))
         return "#version 300 es";
-      else if (gdk_gl_version_greater_equal (&priv->gl_version, &GDK_GL_VERSION_INIT (3, 0)))
-        return "#version 300 es";
       else
         return "#version 100";
     }
@@ -2016,30 +2055,6 @@ gdk_gl_context_get_format_flags (GdkGLContext    *self,
   return priv->memory_flags[format];
 }
 
-gboolean
-gdk_gl_context_has_debug (GdkGLContext *self)
-{
-  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (self);
-
-  return priv->debug_enabled || priv->use_khr_debug;
-}
-
-gboolean
-gdk_gl_context_has_vertex_half_float (GdkGLContext *self)
-{
-  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (self);
-
-  return priv->has_half_float;
-}
-
-gboolean
-gdk_gl_context_has_sync (GdkGLContext *self)
-{
-  GdkGLContextPrivate *priv = gdk_gl_context_get_instance_private (self);
-
-  return priv->has_sync;
-}
-
 /* Return if glGenVertexArrays, glBindVertexArray and glDeleteVertexArrays
  * can be used
  */
@@ -2188,12 +2203,78 @@ gdk_gl_context_import_dmabuf (GdkGLContext    *self,
 
   gdk_display_init_dmabuf (display);
 
-  if (!gdk_dmabuf_formats_contains (display->egl_external_formats, dmabuf->fourcc, dmabuf->modifier))
+  if (gdk_dmabuf_formats_contains (display->egl_dmabuf_formats, dmabuf->fourcc, dmabuf->modifier))
     {
+      /* This is the path for modern drivers that support modifiers */
+
+      if (!gdk_dmabuf_formats_contains (display->egl_external_formats, dmabuf->fourcc, dmabuf->modifier))
+        {
+          texture_id = gdk_gl_context_import_dmabuf_for_target (self,
+                                                                width, height,
+                                                                dmabuf,
+                                                                GL_TEXTURE_2D);
+          if (texture_id == 0)
+            {
+              GDK_DISPLAY_DEBUG (display, DMABUF,
+                                 "Import of %dx%d %.4s:%#" G_GINT64_MODIFIER "x dmabuf failed",
+                                 width, height,
+                                 (char *) &dmabuf->fourcc, dmabuf->modifier);
+              return 0;
+            }
+
+          GDK_DISPLAY_DEBUG (display, DMABUF,
+                             "Imported %dx%d %.4s:%#" G_GINT64_MODIFIER "x dmabuf as GL_TEXTURE_2D texture",
+                             width, height,
+                             (char *) &dmabuf->fourcc, dmabuf->modifier);
+          *external = FALSE;
+          return texture_id;
+        }
+
+      if (!gdk_gl_context_get_use_es (self))
+        {
+          GDK_DISPLAY_DEBUG (display, DMABUF,
+                             "Can't import external_only %.4s:%#" G_GINT64_MODIFIER "x outside of GLES",
+                             (char *) &dmabuf->fourcc, dmabuf->modifier);
+          return 0;
+        }
+
       texture_id = gdk_gl_context_import_dmabuf_for_target (self,
                                                             width, height,
                                                             dmabuf,
-                                                            GL_TEXTURE_2D);
+                                                            GL_TEXTURE_EXTERNAL_OES);
+      if (texture_id == 0)
+        {
+          GDK_DISPLAY_DEBUG (display, DMABUF,
+                             "Import of external_only %dx%d %.4s:%#" G_GINT64_MODIFIER "x dmabuf failed",
+                             width, height,
+                             (char *) &dmabuf->fourcc, dmabuf->modifier);
+          return 0;
+        }
+
+      GDK_DISPLAY_DEBUG (display, DMABUF,
+                         "Imported %dx%d %.4s:%#" G_GINT64_MODIFIER "x dmabuf as GL_TEXTURE_EXTERNAL_OES texture",
+                         width, height,
+                         (char *) &dmabuf->fourcc, dmabuf->modifier);
+      *external = TRUE;
+      return texture_id;
+    }
+  else
+    {
+      /* This is the opportunistic path.
+       * We hit it both for drivers that do not support modifiers as well as for dmabufs
+       * that the driver did not explicitly advertise. */
+      int target;
+
+      if (gdk_gl_context_get_use_es (self))
+        target = GL_TEXTURE_EXTERNAL_OES;
+      else
+        target = GL_TEXTURE_2D;
+
+      texture_id = gdk_gl_context_import_dmabuf_for_target (self,
+                                                            width, height,
+                                                            dmabuf,
+                                                            target);
+
       if (texture_id == 0)
         {
           GDK_DISPLAY_DEBUG (display, DMABUF,
@@ -2204,40 +2285,13 @@ gdk_gl_context_import_dmabuf (GdkGLContext    *self,
         }
 
       GDK_DISPLAY_DEBUG (display, DMABUF,
-                         "Imported %dx%d %.4s:%#" G_GINT64_MODIFIER "x dmabuf as GL_TEXTURE_2D texture",
+                         "Imported %dx%d %.4s:%#" G_GINT64_MODIFIER "x dmabuf as %s texture",
                          width, height,
-                         (char *) &dmabuf->fourcc, dmabuf->modifier);
-      *external = FALSE;
+                         (char *) &dmabuf->fourcc, dmabuf->modifier,
+                         target == GL_TEXTURE_EXTERNAL_OES ? "GL_TEXTURE_EXTERNAL_OES" : "GL_TEXTURE_2D");
+      *external = target == GL_TEXTURE_EXTERNAL_OES;
       return texture_id;
     }
-
-  if (!gdk_gl_context_get_use_es (self))
-    {
-      GDK_DISPLAY_DEBUG (display, DMABUF,
-                         "Can't import external_only %.4s:%#" G_GINT64_MODIFIER "x outside of GLES",
-                         (char *) &dmabuf->fourcc, dmabuf->modifier);
-      return 0;
-    }
-
-  texture_id = gdk_gl_context_import_dmabuf_for_target (self,
-                                                        width, height,
-                                                        dmabuf,
-                                                        GL_TEXTURE_EXTERNAL_OES);
-  if (texture_id == 0)
-    {
-      GDK_DISPLAY_DEBUG (display, DMABUF,
-                         "Import of external_only %dx%d %.4s:%#" G_GINT64_MODIFIER "x dmabuf failed",
-                         width, height,
-                         (char *) &dmabuf->fourcc, dmabuf->modifier);
-      return 0;
-    }
-
-  GDK_DISPLAY_DEBUG (display, DMABUF,
-                     "Imported %dx%d %.4s:%#" G_GINT64_MODIFIER "x dmabuf as GL_TEXTURE_EXTERNAL_OES texture",
-                     width, height,
-                     (char *) &dmabuf->fourcc, dmabuf->modifier);
-  *external = TRUE;
-  return texture_id;
 }
 
 gboolean
