@@ -23,6 +23,8 @@
 #include "gdkvulkancontext.h"
 
 #include "gdkdebugprivate.h"
+#include "gdkdmabufprivate.h"
+#include "gdkdmabufdownloaderprivate.h"
 #include "gdkdrawcontextprivate.h"
 #include "gdkenums.h"
 
@@ -55,6 +57,8 @@ struct _GdkVulkanContextClass
 
 #ifdef GDK_RENDERING_VULKAN
 
+const char *            gdk_vulkan_strerror                         (VkResult           result);
+
 static inline VkResult
 gdk_vulkan_handle_result (VkResult    res,
                           const char *called_function)
@@ -69,15 +73,33 @@ gdk_vulkan_handle_result (VkResult    res,
 
 #define GDK_VK_CHECK(func, ...) gdk_vulkan_handle_result (func (__VA_ARGS__), G_STRINGIFY (func))
 
-gboolean                gdk_display_ref_vulkan                          (GdkDisplay            *display,
+gboolean                gdk_display_init_vulkan                         (GdkDisplay            *display,
                                                                          GError               **error);
+void                    gdk_display_ref_vulkan                          (GdkDisplay            *display);
 void                    gdk_display_unref_vulkan                        (GdkDisplay            *display);
 
+#ifdef HAVE_DMABUF
+GdkDmabufDownloader *   gdk_vulkan_get_dmabuf_downloader                (GdkDisplay            *display,
+                                                                         GdkDmabufFormatsBuilder *builder);
+#endif
+
 VkShaderModule          gdk_display_get_vk_shader_module                (GdkDisplay            *display,
                                                                          const char            *resource_name);
 
-VkPipelineCache         gdk_vulkan_context_get_pipeline_cache           (GdkVulkanContext      *self);
-void                    gdk_vulkan_context_pipeline_cache_updated       (GdkVulkanContext      *self);
+void                    gdk_display_vulkan_pipeline_cache_updated       (GdkDisplay            *display);
+
+VkInstance              gdk_vulkan_context_get_instance                 (GdkVulkanContext      *context);
+VkPhysicalDevice        gdk_vulkan_context_get_physical_device          (GdkVulkanContext      *context);
+VkDevice                gdk_vulkan_context_get_device                   (GdkVulkanContext      *context);
+VkQueue                 gdk_vulkan_context_get_queue                    (GdkVulkanContext      *context);
+uint32_t                gdk_vulkan_context_get_queue_family_index       (GdkVulkanContext      *context);
+VkFormat                gdk_vulkan_context_get_image_format             (GdkVulkanContext      *context);
+GdkMemoryFormat         gdk_vulkan_context_get_memory_format            (GdkVulkanContext      *context);
+uint32_t                gdk_vulkan_context_get_n_images                 (GdkVulkanContext      *context);
+VkImage                 gdk_vulkan_context_get_image                    (GdkVulkanContext      *context,
+                                                                         guint                  id);
+uint32_t                gdk_vulkan_context_get_draw_index               (GdkVulkanContext      *context);
+VkSemaphore             gdk_vulkan_context_get_draw_semaphore           (GdkVulkanContext      *context);
 
 GdkMemoryFormat         gdk_vulkan_context_get_offscreen_format         (GdkVulkanContext      *context,
                                                                          GdkMemoryDepth         depth);
