@@ -108,14 +108,14 @@ static const GtkCssValueClass GTK_CSS_VALUE_CORNER = {
 };
 
 static GtkCssValue corner_singletons[] = {
-  { &GTK_CSS_VALUE_CORNER, 1, TRUE, FALSE, NULL, NULL },
-  { &GTK_CSS_VALUE_CORNER, 1, TRUE, FALSE, NULL, NULL },
-  { &GTK_CSS_VALUE_CORNER, 1, TRUE, FALSE, NULL, NULL },
-  { &GTK_CSS_VALUE_CORNER, 1, TRUE, FALSE, NULL, NULL },
-  { &GTK_CSS_VALUE_CORNER, 1, TRUE, FALSE, NULL, NULL },
-  { &GTK_CSS_VALUE_CORNER, 1, TRUE, FALSE, NULL, NULL },
-  { &GTK_CSS_VALUE_CORNER, 1, TRUE, FALSE, NULL, NULL },
-  { &GTK_CSS_VALUE_CORNER, 1, TRUE, FALSE, NULL, NULL },
+  { &GTK_CSS_VALUE_CORNER, 1, 1, 0, 0, NULL, NULL },
+  { &GTK_CSS_VALUE_CORNER, 1, 1, 0, 0, NULL, NULL },
+  { &GTK_CSS_VALUE_CORNER, 1, 1, 0, 0, NULL, NULL },
+  { &GTK_CSS_VALUE_CORNER, 1, 1, 0, 0, NULL, NULL },
+  { &GTK_CSS_VALUE_CORNER, 1, 1, 0, 0, NULL, NULL },
+  { &GTK_CSS_VALUE_CORNER, 1, 1, 0, 0, NULL, NULL },
+  { &GTK_CSS_VALUE_CORNER, 1, 1, 0, 0, NULL, NULL },
+  { &GTK_CSS_VALUE_CORNER, 1, 1, 0, 0, NULL, NULL },
 };
 
 static inline void
