@@ -29,6 +29,8 @@
 #include "gtkstyleproviderprivate.h"
 #include "gtksymbolicpaintable.h"
 #include "gtkiconthemeprivate.h"
+#include "gtkcsscolorvalueprivate.h"
+#include "gtkcsspalettevalueprivate.h"
 
 G_DEFINE_TYPE (GtkCssImageIconTheme, _gtk_css_image_icon_theme, GTK_TYPE_CSS_IMAGE)
 
@@ -50,6 +52,7 @@ gtk_css_image_icon_theme_snapshot (GtkCssImage *image,
   double icon_width, icon_height;
   int size;
   double x, y;
+  GdkRGBA colors[4];
 
   size = floor (MIN (width, height));
   if (size <= 0)
@@ -89,11 +92,15 @@ gtk_css_image_icon_theme_snapshot (GtkCssImage *image,
       gtk_snapshot_save (snapshot);
       gtk_snapshot_translate (snapshot, &GRAPHENE_POINT_INIT (x, y));
     }
+
+  for (guint i = 0; i < 4; i++)
+    colors[i] = *gtk_css_color_value_get_rgba (icon_theme->colors[i]);
+
   gtk_symbolic_paintable_snapshot_symbolic (GTK_SYMBOLIC_PAINTABLE (icon),
                                             snapshot,
                                             icon_width,
                                             icon_height,
-                                            icon_theme->colors,
+                                            colors,
                                             G_N_ELEMENTS (icon_theme->colors));
   if (x != 0 || y != 0)
     gtk_snapshot_restore (snapshot);
@@ -146,6 +153,7 @@ gtk_css_image_icon_theme_compute (GtkCssImage          *image,
   GtkCssImageIconTheme *copy;
   GtkSettings *settings;
   GdkDisplay *display;
+  const char *names[4] = { NULL, "success", "warning", "error" };
 
   copy = g_object_new (GTK_TYPE_CSS_IMAGE_ICON_THEME, NULL);
   copy->name = g_strdup (icon_theme->name);
@@ -154,7 +162,18 @@ gtk_css_image_icon_theme_compute (GtkCssImage          *image,
   copy->icon_theme = gtk_icon_theme_get_for_display (display);
   copy->serial = gtk_icon_theme_get_serial (copy->icon_theme);
   copy->scale = gtk_style_provider_get_scale (context->provider);
-  gtk_css_style_lookup_symbolic_colors (context->style, copy->colors);
+
+  for (guint i = 0; i < 4; i++)
+    {
+      GtkCssValue *color = NULL;
+
+      if (names[i])
+        color = gtk_css_palette_value_get_color (context->style->core->icon_palette, names[i]);
+      if (color)
+        copy->colors[i] = gtk_css_value_ref (color);
+      else
+        copy->colors[i] = gtk_css_value_ref (context->style->core->color);
+    }
 
   return GTK_CSS_IMAGE (copy);
 }
@@ -180,10 +199,52 @@ gtk_css_image_icon_theme_dispose (GObject *object)
   icon_theme->name = NULL;
 
   g_clear_object (&icon_theme->cached_icon);
+  g_clear_pointer (&icon_theme->colors[0], gtk_css_value_unref);
+  g_clear_pointer (&icon_theme->colors[1], gtk_css_value_unref);
+  g_clear_pointer (&icon_theme->colors[2], gtk_css_value_unref);
+  g_clear_pointer (&icon_theme->colors[3], gtk_css_value_unref);
 
   G_OBJECT_CLASS (_gtk_css_image_icon_theme_parent_class)->dispose (object);
 }
 
+static gboolean
+gtk_css_image_icon_theme_contains_current_color (GtkCssImage *image)
+{
+  GtkCssImageIconTheme *icon_theme = GTK_CSS_IMAGE_ICON_THEME (image);
+
+  for (guint i = 0; i < 4; i++)
+    {
+      if (!icon_theme->colors[i] ||
+          gtk_css_value_contains_current_color (icon_theme->colors[i]))
+        return TRUE;
+    }
+
+  return FALSE;
+}
+
+static GtkCssImage *
+gtk_css_image_icon_theme_resolve (GtkCssImage          *image,
+                                  GtkCssComputeContext *context,
+                                  GtkCssValue          *current)
+{
+  GtkCssImageIconTheme *icon_theme = GTK_CSS_IMAGE_ICON_THEME (image);
+  GtkCssImageIconTheme *copy;
+
+  if (!gtk_css_image_icon_theme_contains_current_color (image))
+    return g_object_ref (image);
+
+  copy = g_object_new (GTK_TYPE_CSS_IMAGE_ICON_THEME, NULL);
+  copy->name = g_strdup (icon_theme->name);
+  copy->icon_theme = icon_theme->icon_theme;
+  copy->serial = icon_theme->serial;
+  copy->scale = icon_theme->scale;
+
+  for (guint i = 0; i < 4; i++)
+    copy->colors[i] = gtk_css_color_value_resolve (icon_theme->colors[i], context, current);
+
+  return GTK_CSS_IMAGE (copy);
+}
+
 static void
 _gtk_css_image_icon_theme_class_init (GtkCssImageIconThemeClass *klass)
 {
@@ -196,7 +257,8 @@ _gtk_css_image_icon_theme_class_init (GtkCssImageIconThemeClass *klass)
   image_class->print = gtk_css_image_icon_theme_print;
   image_class->compute = gtk_css_image_icon_theme_compute;
   image_class->equal = gtk_css_image_icon_theme_equal;
-
+  image_class->contains_current_color = gtk_css_image_icon_theme_contains_current_color;
+  image_class->resolve = gtk_css_image_icon_theme_resolve;
   object_class->dispose = gtk_css_image_icon_theme_dispose;
 }
 
