@@ -134,7 +134,7 @@ static const GtkCssValueClass GTK_CSS_VALUE_IMAGE = {
 GtkCssValue *
 _gtk_css_image_value_new (GtkCssImage *image)
 {
-  static GtkCssValue image_none_singleton = { &GTK_CSS_VALUE_IMAGE, 1, TRUE, FALSE, NULL };
+  static GtkCssValue image_none_singleton = { &GTK_CSS_VALUE_IMAGE, 1, 1, 0, 0, NULL };
   GtkCssValue *value;
 
   if (image == NULL)
@@ -143,6 +143,7 @@ _gtk_css_image_value_new (GtkCssImage *image)
   value = gtk_css_value_new (GtkCssValue, &GTK_CSS_VALUE_IMAGE);
   value->image = image;
   value->is_computed = gtk_css_image_is_computed (image);
+  value->contains_current_color = gtk_css_image_contains_current_color (image);
 
   return value;
 }
@@ -155,3 +156,13 @@ _gtk_css_image_value_get_image (const GtkCssValue *value)
   return value->image;
 }
 
+GtkCssValue *
+gtk_css_image_value_resolve (GtkCssValue          *value,
+                             GtkCssComputeContext *context,
+                             GtkCssValue          *current_color)
+{
+  if (!gtk_css_value_contains_current_color (value))
+    return gtk_css_value_ref (value);
+
+  return _gtk_css_image_value_new (gtk_css_image_resolve (_gtk_css_image_value_get_image (value), context, current_color));
+}
