@@ -30,8 +30,12 @@ GtkCssValue *   gtk_css_palette_value_new_default       (void);
 
 GtkCssValue *   gtk_css_palette_value_parse             (GtkCssParser        *parser);
 
-const GdkRGBA * gtk_css_palette_value_get_color         (GtkCssValue         *value,
+GtkCssValue *   gtk_css_palette_value_get_color         (GtkCssValue         *value,
                                                          const char          *color_name);
 
+GtkCssValue *   gtk_css_palette_value_resolve           (GtkCssValue          *value,
+                                                         GtkCssComputeContext *context,
+                                                         GtkCssValue          *current_color);
+
 G_END_DECLS
 
