@@ -26,11 +26,13 @@
 #include "gtkcssarrayvalueprivate.h"
 #include "gtkcsscustompropertypoolprivate.h"
 #include "gtkcssenumvalueprivate.h"
+#include "gtkcssimagevalueprivate.h"
 #include "gtkcssinheritvalueprivate.h"
 #include "gtkcssinitialvalueprivate.h"
 #include "gtkcssnumbervalueprivate.h"
 #include "gtkcsscolorvalueprivate.h"
 #include "gtkcsspalettevalueprivate.h"
+#include "gtkcssshadowvalueprivate.h"
 #include "gtkcssshorthandpropertyprivate.h"
 #include "gtkcssstringvalueprivate.h"
 #include "gtkcssfontvariationsvalueprivate.h"
@@ -41,6 +43,7 @@
 #include "gtkstyleanimationprivate.h"
 #include "gtkstylepropertyprivate.h"
 #include "gtkstyleproviderprivate.h"
+#include "gtkcssvaluesprivate.h"
 
 G_DEFINE_ABSTRACT_TYPE (GtkCssStyle, gtk_css_style, G_TYPE_OBJECT)
 
@@ -74,6 +77,7 @@ gtk_css_style_finalize (GObject *object)
   gtk_css_values_unref ((GtkCssValues *)style->transition);
   gtk_css_values_unref ((GtkCssValues *)style->size);
   gtk_css_values_unref ((GtkCssValues *)style->other);
+  gtk_css_values_unref ((GtkCssValues *)style->used);
 
   if (style->variables)
     gtk_css_variable_set_unref (style->variables);
@@ -100,6 +104,59 @@ gtk_css_style_init (GtkCssStyle *style)
 GtkCssValue *
 gtk_css_style_get_value (GtkCssStyle *style,
                          guint        id)
+{
+  return gtk_css_style_get_used_value (style, id);
+}
+
+GtkCssValue *
+gtk_css_style_get_used_value (GtkCssStyle *style,
+                              guint        id)
+{
+  switch (id)
+    {
+    case GTK_CSS_PROPERTY_COLOR:
+      return style->used->color;
+    case GTK_CSS_PROPERTY_ICON_PALETTE:
+      return style->used->icon_palette;
+    case GTK_CSS_PROPERTY_BACKGROUND_COLOR:
+      return style->used->background_color;
+    case GTK_CSS_PROPERTY_BOX_SHADOW:
+      return style->used->box_shadow;
+    case GTK_CSS_PROPERTY_BACKGROUND_IMAGE:
+      return style->used->background_image;
+    case GTK_CSS_PROPERTY_BORDER_TOP_COLOR:
+      return style->used->border_top_color;
+    case GTK_CSS_PROPERTY_BORDER_RIGHT_COLOR:
+      return style->used->border_right_color;
+    case GTK_CSS_PROPERTY_BORDER_BOTTOM_COLOR:
+      return style->used->border_bottom_color;
+    case GTK_CSS_PROPERTY_BORDER_LEFT_COLOR:
+      return style->used->border_left_color;
+    case GTK_CSS_PROPERTY_BORDER_IMAGE_SOURCE:
+      return style->used->border_image_source;
+    case GTK_CSS_PROPERTY_ICON_SHADOW:
+      return style->used->icon_shadow;
+    case GTK_CSS_PROPERTY_OUTLINE_COLOR:
+      return style->used->outline_color;
+    case GTK_CSS_PROPERTY_CARET_COLOR:
+      return style->used->caret_color;
+    case GTK_CSS_PROPERTY_SECONDARY_CARET_COLOR:
+      return style->used->secondary_caret_color;
+    case GTK_CSS_PROPERTY_TEXT_SHADOW:
+      return style->used->text_shadow;
+    case GTK_CSS_PROPERTY_TEXT_DECORATION_COLOR:
+      return style->used->text_decoration_color;
+    case GTK_CSS_PROPERTY_ICON_SOURCE:
+      return style->used->icon_source;
+
+    default:
+      return gtk_css_style_get_computed_value (style, id);
+    }
+}
+
+GtkCssValue *
+gtk_css_style_get_computed_value (GtkCssStyle *style,
+                                  guint        id)
 {
   switch (id)
     {
@@ -128,7 +185,7 @@ gtk_css_style_get_value (GtkCssStyle *style,
     case GTK_CSS_PROPERTY_TEXT_DECORATION_LINE:
       return style->font_variant->text_decoration_line;
     case GTK_CSS_PROPERTY_TEXT_DECORATION_COLOR:
-      return style->font_variant->text_decoration_color ? style->font_variant->text_decoration_color : style->core->color;
+      return style->font_variant->text_decoration_color;
     case GTK_CSS_PROPERTY_TEXT_DECORATION_STYLE:
       return style->font_variant->text_decoration_style;
     case GTK_CSS_PROPERTY_TEXT_TRANSFORM:
@@ -206,15 +263,15 @@ gtk_css_style_get_value (GtkCssStyle *style,
     case GTK_CSS_PROPERTY_BACKGROUND_POSITION:
       return style->background->background_position;
     case GTK_CSS_PROPERTY_BORDER_TOP_COLOR:
-      return style->border->border_top_color ? style->border->border_top_color : style->core->color;
+      return style->border->border_top_color;
     case GTK_CSS_PROPERTY_BORDER_RIGHT_COLOR:
-      return style->border->border_right_color ? style->border->border_right_color : style->core->color;
+      return style->border->border_right_color;
     case GTK_CSS_PROPERTY_BORDER_BOTTOM_COLOR:
-      return style->border->border_bottom_color ? style->border->border_bottom_color : style->core->color;
+      return style->border->border_bottom_color;
     case GTK_CSS_PROPERTY_BORDER_LEFT_COLOR:
-      return style->border->border_left_color ? style->border->border_left_color: style->core->color;
+      return style->border->border_left_color;
     case GTK_CSS_PROPERTY_OUTLINE_COLOR:
-      return style->outline->outline_color ? style->outline->outline_color : style->core->color;
+      return style->outline->outline_color;
     case GTK_CSS_PROPERTY_BACKGROUND_REPEAT:
       return style->background->background_repeat;
     case GTK_CSS_PROPERTY_BACKGROUND_IMAGE:
@@ -280,9 +337,9 @@ gtk_css_style_get_value (GtkCssStyle *style,
     case GTK_CSS_PROPERTY_FILTER:
       return style->other->filter;
     case GTK_CSS_PROPERTY_CARET_COLOR:
-      return style->font->caret_color ? style->font->caret_color : style->core->color;
+      return style->font->caret_color;
     case GTK_CSS_PROPERTY_SECONDARY_CARET_COLOR:
-      return style->font->secondary_caret_color ? style->font->secondary_caret_color : style->core->color;
+      return style->font->secondary_caret_color;
     case GTK_CSS_PROPERTY_FONT_FEATURE_SETTINGS:
       return style->font->font_feature_settings;
     case GTK_CSS_PROPERTY_FONT_VARIATION_SETTINGS:
@@ -355,16 +412,23 @@ gtk_css_style_print (GtkCssStyle *style,
     {
       GtkCssSection *section;
       GtkCssStyleProperty *prop;
-      GtkCssValue *value;
+      GtkCssValue *value, *computed, *initial;
       const char *name;
 
-      section = gtk_css_style_get_section (style, i);
-      if (!section && skip_initial)
-        continue;
-
       prop = _gtk_css_style_property_lookup_by_id (i);
       name = _gtk_style_property_get_name (GTK_STYLE_PROPERTY (prop));
-      value = gtk_css_style_get_value (style, i);
+      computed = gtk_css_style_get_computed_value (style, i);
+      value = gtk_css_style_get_used_value (style, i);
+      initial = _gtk_css_style_property_get_initial_value (prop);
+
+      section = gtk_css_style_get_section (style, i);
+      if (skip_initial)
+        {
+          if (!section &&
+              (computed == initial ||
+               !gtk_css_value_contains_current_color (computed)))
+            continue;
+        }
 
       g_string_append_printf (string, "%*s%s: ", indent, "", name);
       gtk_css_value_print (value, string);
@@ -647,20 +711,22 @@ gtk_css_style_get_pango_attributes (GtkCssStyle *style)
   GtkTextDecorationStyle decoration_style;
   const GdkRGBA *color;
   const GdkRGBA *decoration_color;
+  gboolean has_decoration_color;
   double letter_spacing;
 
   /* text-decoration */
   decoration_line = _gtk_css_text_decoration_line_value_get (style->font_variant->text_decoration_line);
   decoration_style = _gtk_css_text_decoration_style_value_get (style->font_variant->text_decoration_style);
-  color = gtk_css_color_value_get_rgba (style->core->color);
-  decoration_color = gtk_css_color_value_get_rgba (style->font_variant->text_decoration_color
-                                                   ? style->font_variant->text_decoration_color
-                                                   : style->core->color);
+
+  color = gtk_css_color_value_get_rgba (style->used->color);
+  decoration_color = gtk_css_color_value_get_rgba (style->used->text_decoration_color);
+
+  has_decoration_color = !gdk_rgba_equal (color, decoration_color);
 
   if (decoration_line & GTK_CSS_TEXT_DECORATION_LINE_UNDERLINE)
     {
       attrs = add_pango_attr (attrs, pango_attr_underline_new (get_pango_underline_from_style (decoration_style)));
-      if (!gdk_rgba_equal (color, decoration_color))
+      if (has_decoration_color)
         attrs = add_pango_attr (attrs, pango_attr_underline_color_new (decoration_color->red * 65535. + 0.5,
                                                                        decoration_color->green * 65535. + 0.5,
                                                                        decoration_color->blue * 65535. + 0.5));
@@ -668,7 +734,7 @@ gtk_css_style_get_pango_attributes (GtkCssStyle *style)
   if (decoration_line & GTK_CSS_TEXT_DECORATION_LINE_OVERLINE)
     {
       attrs = add_pango_attr (attrs, pango_attr_overline_new (get_pango_overline_from_style (decoration_style)));
-      if (!gdk_rgba_equal (color, decoration_color))
+      if (has_decoration_color)
         attrs = add_pango_attr (attrs, pango_attr_overline_color_new (decoration_color->red * 65535. + 0.5,
                                                                       decoration_color->green * 65535. + 0.5,
                                                                       decoration_color->blue * 65535. + 0.5));
@@ -676,7 +742,7 @@ gtk_css_style_get_pango_attributes (GtkCssStyle *style)
   if (decoration_line & GTK_CSS_TEXT_DECORATION_LINE_LINE_THROUGH)
     {
       attrs = add_pango_attr (attrs, pango_attr_strikethrough_new (TRUE));
-      if (!gdk_rgba_equal (color, decoration_color))
+      if (!has_decoration_color)
         attrs = add_pango_attr (attrs, pango_attr_strikethrough_color_new (decoration_color->red * 65535. + 0.5,
                                                                            decoration_color->green * 65535. + 0.5,
                                                                            decoration_color->blue * 65535. + 0.5));
@@ -805,24 +871,22 @@ void
 gtk_css_style_lookup_symbolic_colors (GtkCssStyle *style,
                                       GdkRGBA      color_out[4])
 {
-  GtkCssValue *palette, *color;
   const char *names[4] = {
     [GTK_SYMBOLIC_COLOR_ERROR] = "error",
     [GTK_SYMBOLIC_COLOR_WARNING] = "warning",
     [GTK_SYMBOLIC_COLOR_SUCCESS] = "success"
   };
 
-  color = style->core->color;
-  palette = style->core->icon_palette;
-  color_out[GTK_SYMBOLIC_COLOR_FOREGROUND] = *gtk_css_color_value_get_rgba (color);
+  color_out[GTK_SYMBOLIC_COLOR_FOREGROUND] = *gtk_css_color_value_get_rgba (style->used->color);
 
   for (gsize i = 1; i < 4; i++)
     {
-      const GdkRGBA *lookup;
+      GtkCssValue *lookup;
+
+      lookup = gtk_css_palette_value_get_color (style->used->icon_palette, names[i]);
 
-      lookup = gtk_css_palette_value_get_color (palette, names[i]);
       if (lookup)
-        color_out[i] = *lookup;
+        color_out[i] = *gtk_css_color_value_get_rgba (lookup);
       else
         color_out[i] = color_out[GTK_SYMBOLIC_COLOR_FOREGROUND];
     }
@@ -841,7 +905,8 @@ static const int values_size[] = {
   sizeof (GtkCssAnimationValues),
   sizeof (GtkCssTransitionValues),
   sizeof (GtkCssSizeValues),
-  sizeof (GtkCssOtherValues)
+  sizeof (GtkCssOtherValues),
+  sizeof (GtkCssUsedValues)
 };
 
 #define TYPE_INDEX(type) ((type) - ((type) % 2))
@@ -932,3 +997,100 @@ gtk_css_style_list_custom_properties (GtkCssStyle *style)
 
   return NULL;
 }
+
+GtkCssValue *
+gtk_css_style_resolve_used_value (GtkCssStyle          *style,
+                                  GtkCssValue          *value,
+                                  guint                 id,
+                                  GtkCssComputeContext *context)
+{
+  GtkCssValue *used;
+
+  switch (id)
+    {
+    case GTK_CSS_PROPERTY_COLOR:
+      {
+        GtkCssValue *current;
+
+        if (context->parent_style)
+          current = context->parent_style->used->color;
+        else
+          current = _gtk_css_style_property_get_initial_value (_gtk_css_style_property_lookup_by_id (GTK_CSS_PROPERTY_COLOR));
+
+        used = gtk_css_color_value_resolve (value, context, current);
+      }
+      break;
+
+    case GTK_CSS_PROPERTY_BACKGROUND_COLOR:
+    case GTK_CSS_PROPERTY_TEXT_DECORATION_COLOR:
+    case GTK_CSS_PROPERTY_BORDER_TOP_COLOR:
+    case GTK_CSS_PROPERTY_BORDER_RIGHT_COLOR:
+    case GTK_CSS_PROPERTY_BORDER_BOTTOM_COLOR:
+    case GTK_CSS_PROPERTY_BORDER_LEFT_COLOR:
+    case GTK_CSS_PROPERTY_OUTLINE_COLOR:
+    case GTK_CSS_PROPERTY_CARET_COLOR:
+    case GTK_CSS_PROPERTY_SECONDARY_CARET_COLOR:
+      used = gtk_css_color_value_resolve (value, context, style->used->color);
+      break;
+
+    case GTK_CSS_PROPERTY_BOX_SHADOW:
+    case GTK_CSS_PROPERTY_TEXT_SHADOW:
+    case GTK_CSS_PROPERTY_ICON_SHADOW:
+      used = gtk_css_shadow_value_resolve (value, context, style->used->color);
+      break;
+
+    case GTK_CSS_PROPERTY_ICON_PALETTE:
+      used = gtk_css_palette_value_resolve (value, context, style->used->color);
+      break;
+
+    case GTK_CSS_PROPERTY_BACKGROUND_IMAGE:
+    case GTK_CSS_PROPERTY_ICON_SOURCE:
+    case GTK_CSS_PROPERTY_BORDER_IMAGE_SOURCE:
+      used = gtk_css_image_value_resolve (value, context, style->used->color);
+      break;
+
+    default:
+      return NULL;
+    }
+
+  g_assert (!gtk_css_value_contains_current_color (used));
+
+  return used;
+}
+
+static inline void
+gtk_css_take_value (GtkCssValue **variable,
+                    GtkCssValue  *value)
+{
+  if (*variable)
+    gtk_css_value_unref (*variable);
+  *variable = value;
+}
+
+void
+gtk_css_style_resolve_used_values (GtkCssStyle          *style,
+                                   GtkCssComputeContext *context)
+{
+  GtkCssValue **values;
+
+  if (style->used)
+    gtk_css_values_unref ((GtkCssValues *) style->used);
+
+  style->used = (GtkCssUsedValues *) gtk_css_values_new (GTK_CSS_USED_VALUES);
+  values = &style->used->color;
+
+  for (guint i = 0; i < G_N_ELEMENTS (used_props); i++)
+    {
+      guint id = used_props[i];
+      GtkCssValue *value, *used;
+
+      value = gtk_css_style_get_computed_value (style, id);
+
+      if (gtk_css_value_contains_current_color (value))
+        used = gtk_css_style_resolve_used_value (style, value, id, context);
+      else
+        used = gtk_css_value_ref (value);
+
+      gtk_css_take_value (&values[i], used);
+    }
+}
