@@ -67,6 +67,17 @@
  * </section>
  * ```
  *
+ * # Shortcuts and Gestures
+ *
+ * `GtkPopover` supports the following keyboard shortcuts:
+ *
+ * - <kbd>Escape</kbd> closes the popover.
+ * - <kbd>Alt</kbd> makes the mnemonics visible.
+ *
+ * The following signals have default keybindings:
+ *
+ * - [signal@Gtk.Popover::activate-default]
+ *
  * # CSS nodes
  *
  * ```
@@ -377,6 +388,7 @@ update_popover_layout (GtkPopover     *popover,
   GdkRectangle final_rect;
   gboolean flipped_x;
   gboolean flipped_y;
+  gboolean attachment_point_changed;
   GdkPopup *popup = GDK_POPUP (priv->surface);
   GtkPositionType position;
 
@@ -401,6 +413,9 @@ update_popover_layout (GtkPopover     *popover,
     did_flip_vertically (gdk_popup_layout_get_surface_anchor (layout),
                          gdk_popup_get_surface_anchor (popup));
 
+  attachment_point_changed = final_rect.x != priv->final_rect.x ||
+                             final_rect.y != priv->final_rect.y;
+
   priv->final_rect = final_rect;
 
   position = priv->final_position;
@@ -426,7 +441,8 @@ update_popover_layout (GtkPopover     *popover,
 
   if (priv->final_position != position ||
       priv->final_rect.width != width ||
-      priv->final_rect.height != height)
+      priv->final_rect.height != height ||
+      attachment_point_changed)
     {
       gtk_widget_queue_allocate (GTK_WIDGET (popover));
       g_clear_pointer (&priv->arrow_render_node, gsk_render_node_unref);
@@ -693,6 +709,14 @@ gtk_popover_native_layout (GtkNative *native,
   GtkPopover *popover = GTK_POPOVER (native);
   GtkPopoverPrivate *priv = gtk_popover_get_instance_private (popover);
   GtkWidget *widget = GTK_WIDGET (popover);
+  GtkRequisition min, nat;
+
+  gtk_widget_get_preferred_size (widget, &min, &nat);
+  if (width < min.width || height < min.height)
+    {
+      gtk_popover_popdown (popover);
+      return;
+    }
 
   update_popover_layout (popover, gdk_popup_layout_ref (priv->layout), width, height);
 
@@ -1627,7 +1651,7 @@ create_arrow_render_node (GtkPopover *popover)
     {
       const GdkRGBA *border_color;
 
-      border_color = gtk_css_color_value_get_rgba (style->border->border_left_color ? style->border->border_left_color : style->core->color);
+      border_color = gtk_css_color_value_get_rgba (style->used->border_left_color);
 
       gtk_popover_apply_tail_path (popover, cr);
       gdk_cairo_set_source_rgba (cr, border_color);
@@ -1960,6 +1984,8 @@ gtk_popover_class_init (GtkPopoverClass *klass)
    * Emitted whend the user activates the default widget.
    *
    * This is a [keybinding signal](class.SignalAction.html).
+   *
+   * The default binding for this signal is <kbd>Enter</kbd>.
    */
   signals[ACTIVATE_DEFAULT] =
     g_signal_new (I_("activate-default"),
