@@ -5,7 +5,7 @@ window_role (void)
 {
   GtkWidget *window = gtk_window_new ();
 
-  gtk_test_accessible_assert_role (window, GTK_ACCESSIBLE_ROLE_WINDOW);
+  gtk_test_accessible_assert_role (window, GTK_ACCESSIBLE_ROLE_APPLICATION);
 
   gtk_window_destroy (GTK_WINDOW (window));
 }
@@ -19,7 +19,7 @@ window_state (void)
 
   gtk_test_accessible_assert_state (window, GTK_ACCESSIBLE_STATE_HIDDEN, FALSE);
 
-  gtk_widget_hide (window);
+  gtk_widget_set_visible (window, FALSE);
 
   gtk_test_accessible_assert_state (window, GTK_ACCESSIBLE_STATE_HIDDEN, TRUE);
 
